﻿JebusMiniMapCoords = LibStub("AceAddon-3.0"):NewAddon("JebusMiniMapCoords", "AceEvent-3.0", "AceConsole-3.0");

local JebusMiniMapCoordsFrame

local L = LibStub("AceLocale-3.0"):GetLocale("JebusMiniMapCoords", true)

function JebusMiniMapCoords:OnInitialize()
	self.name = L["MiniMap Coords"]
	self.title = L["MiniMap Coords"]
	self.db = LibStub("AceDB-3.0"):New("JebusMiniMapCoordsDB", { profile = {
			showBackground = true,
			showBorder = true,
			point = "BOTTOM",
			relto = "Minimap",
			relpoint = "BOTTOM",
			bgR = 0,
			bgG = 0,
			bgB = 0,
			bgA = 1,
			txtR = 1,
			txtG = 0.82,
			txtB = 0,
			txtA = 1,
			txtSize = 1,
			highPrecision = true,
			offsetX = 0,
			offsetY = -25,
		}}, true)

	local options = {
		name = L["MiniMap Coords"],
		desc = L["Description"],
		type = 'group',
		args = {
			bBackground = {
				name = L["Show Background"],
				desc = "",
				type = "toggle",
				order = 5,
				get = "GetBackground",
				set = "SetBackground",
			},
			bgColor = {
				name = L["Background color"],
				desc = L["The color of the background frame behind coordinates"],
				type = "color",
				order = 10,
				get = "GetBackgroundColor",
				set = "SetBackgroundColor",
				hasAlpha = true,
			},
			color = {
				name = L["Text color"],
				desc = L["The color of the coordinates"],
				type = "color",
				order = 15,
				get = "GetTextColor",
				set = "SetTextColor",
				hasAlpha = true,
			},
			bBorder = {
				name = L["Show Border"],
				desc = "",
				type = "toggle",
				order = 20,
				get = "GetBorder",
				set = "SetBorder",
			},
			highPrecision = {
				name = L["High precision"],
				desc = L["Show high precision coordinates"],
				type = "toggle",
				get = "GetHighPrecision",
				set = "SetHighPrecision",
			},
			cReset = {
				name = L["Reset Position"],
				desc = "",
				type = "execute",
				order = 999,
				func = "ResetPosition",
			},
		},
		handler = self,
	}
	LibStub("AceConfig-3.0"):RegisterOptionsTable(L["MiniMap Coords"], options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions(L["MiniMap Coords"])
end

function JebusMiniMapCoords:OnEnable()
	if self.frame then
		self.frame:Show()
	else
		self.frame = CreateFrame("Frame", "JebusMiniMapCoordsFrame", UIParent)
		JebusMiniMapCoordsFrame = self.frame

		self.frame:EnableMouse(true)
		self.frame:SetMovable(true)
		--self:Print("point="..(self.db.point or "nil")..", relto="..(self.db.profile.relto or "nil")..", relpoint="..(self.db.profile.relpoint or "nil")..", offsetX="..(self.db.profile.offsetX or "nil")..", offsetY="..(self.db.profile.offsetY or "nil"))
		--[[if self.db.profile.point == "" or self.db.profile.point == nil then
			self.db.profile.point = "BOTTOM"
			self.db.profile.relto = "Minimap"
			self.db.profile.relpoint = "BOTTOM"
			self.db.profile.offsetX = 0
			self.db.profile.offsetY = -25
		end]]
		local relto = self.db.profile.relto
		if relto == "" then relto = nil end
		self.frame:SetPoint(self.db.profile.point, relto, self.db.profile.relpoint, self.db.profile.offsetX, self.db.profile.offsetY)
		self.frame:SetWidth(65)
		self.frame:SetHeight(22)
		self.frame:SetBackdrop({
			bgFile = "Interface/TutorialFrame/TutorialFrameBackground",
			edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
			tile = true,
			tileSize = 16,
			edgeSize = 16,
			insets = { left=5, right = 5, top = 5, bottom = 5 }
		})
		self.frame:SetBackdropColor(self.db.profile.bgR, self.db.profile.bgG, self.db.profile.bgB, self.db.profile.bgA)

		self.frame.loc = self.frame:CreateFontString(nil, "OVERLAY")
		self.frame.loc:SetWidth(65)
		self.frame.loc:SetHeight(22)
		self.frame.loc:SetPoint("TOPLEFT",self.frame, "TOPLEFT")
		self.frame.loc:SetJustifyH("CENTER")
		self.frame.loc:SetJustifyV("CENTER")
		self.frame.loc:SetFontObject(GameFontNormalSmall)
		self.frame.loc:SetTextColor(self.db.profile.txtR, self.db.profile.txtG, self.db.profile.txtB, self.db.profile.txtA)

--		self.frame:SetScript("OnUpdate", JebusMiniMapCoords_OnUpdate)
		self.frame:SetScript("OnUpdate", 
			function()
				self:OnUpdate()
			end
		)
		self.frame:SetScript("OnDragStart",
			function()
				self.frame:StartMoving()
			end
		)
		self.frame:SetScript("OnDragStop",
			function()
				self.frame:StopMovingOrSizing()
				self:SavePosition()
			end
		)
		self.frame:RegisterForDrag("RightButton")
		self.frame:Show()
	end
	self:SetBackground(true, self:GetBackground())
	self:SetBorder(true, self:GetBorder())
end

local formatHighStr, formatLowStr
do
	if ("%.1f"):format(1.1) == "1,1" then
		formatHighStr = "%.1f x %.1f"
		formatLowStr = "%.0f x %.0f"
	else
		formatHighStr = "%.1f, %.1f"
		formatLowStr = "%.0f, %.0f"
	end
end

--local function JebusMiniMapCoords_OnUpdate()
function JebusMiniMapCoords:OnUpdate()
	local x, y = GetPlayerMapPosition("player")
	local formatStr = JebusMiniMapCoords.db.profile.highPrecision and formatHighStr or formatLowStr

	if x == 0 or y == 0 or (IsInInstance() and select(2, IsInInstance()) ~= "pvp") then
		JebusMiniMapCoordsFrame.loc:SetText("---")
	else
		JebusMiniMapCoordsFrame.loc:SetText(string.format(formatStr, (x*100), (y*100)))
	end
end

function JebusMiniMapCoords:GetBackground()
	return self.db.profile.showBackground
end

function JebusMiniMapCoords:SetBackground(tbl, value)
	if value == nil then
		return false
	end
	self.db.profile.showBackground = value
	if self.db.profile.showBackground then
		self.frame:SetBackdropColor(1,1,1,1)
	else
		self.frame:SetBackdropColor(0,0,0,0)
	end
end


function JebusMiniMapCoords:GetBorder()
	return self.db.profile.showBorder
end

function JebusMiniMapCoords:SetBorder(tbl, value)
	if value == nil then
		return false
	end
	self.db.profile.showBorder = value
	if self.db.profile.showBorder then
		self.frame:SetBackdropBorderColor(1,1,1,1)
	else
		self.frame:SetBackdropBorderColor(0,0,0,0)
	end
end

function JebusMiniMapCoords:OnDisable()
	self:UnregisterAllEvents()
	self.frame:Hide()
end

function JebusMiniMapCoords:SavePosition()
	local point, relto, relpoint, xoff, yoff = self.frame:GetPoint(1)
	self.db.profile.point = point
	self.db.profile.relto = relto or ""
	self.db.profile.relpoint = relpoint
	self.db.profile.offsetX = xoff
	self.db.profile.offsetY = yoff
end

function JebusMiniMapCoords:ResetPosition()
	self.frame:ClearAllPoints()
	if(Minimap:IsVisible()) then
		self.frame:SetPoint("BOTTOM", Minimap, "BOTTOM", 0, -25)
	else
		self.frame:SetPoint("CENTER", "", "CENTER", 0, 0)
	end
	self:SavePosition()
end

function JebusMiniMapCoords:GetBackgroundColor()
	return self.db.profile.bgR, self.db.profile.bgG, self.db.profile.bgB, self.db.profile.bgA
end

function JebusMiniMapCoords:SetBackgroundColor(tbl, r, g, b, a)
	self.db.profile.bgR, self.db.profile.bgG, self.db.profile.bgB, self.db.profile.bgA = r, g, b, a
	self.frame:SetBackdropColor(r, g, b, a)
end

function JebusMiniMapCoords:GetTextColor()
	return self.db.profile.txtR, self.db.profile.txtG, self.db.profile.txtB, self.db.profile.txtA
end

function JebusMiniMapCoords:SetTextColor(tbl, r, g, b, a)
	self.db.profile.txtR, self.db.profile.txtG, self.db.profile.txtB, self.db.profile.txtA = r, g, b, a
	self.frame.loc:SetTextColor(r, g, b, a)
end

function JebusMiniMapCoords:GetHighPrecision()
	return self.db.profile.highPrecision
end

function JebusMiniMapCoords:SetHighPrecision(tbl, value)
	if value == nil then
		return false
	end
	self.db.profile.highPrecision = value
end
